package com.javaranch.common ;

import java.util.* ;
import java.io.* ;
import java.sql.* ;
import javax.ejb.* ;
import javax.naming.* ;
import javax.sql.* ;

/** Abstract base class for EntityEJB and SessionEJB. <p>

    This class is intended for use inside an EJB server only! <p> 
    
    Provides database connections and SQL helper methods. <p>
    
    - - - - - - - - - - - - - - - - - <p>
    
    Copyright (c) 1998-2000 Paul Wheaton <p>
    
    You are welcome to do whatever you want to with this source file provided
    that you maintain this comment fragment (between the dashed lines).
    Modify it, change the package name, change the class name ... personal or business 
    use ...  sell it, share it ... add a copyright for the portions you add ... <p>
    
    My goal in giving this away and maintaining the copyright is to hopefully direct 
    developers back to JavaRanch. <p>
    
    The original source can be found at <a href=http://www.javaranch.com>JavaRanch</a> <p>
    
    - - - - - - - - - - - - - - - - - <p>
    
    @author Paul Wheaton
*/
public abstract class EJB
{

    private static final DataSource dataSource ;

    // this static block of code is run sometime after the class is first loaded and before the class is first used.
    static    
    {
        // get dataSource    
        try
        {
            Context jndi = new InitialContext();
            dataSource = (DataSource)jndi.lookup( "java:comp/env/jdbc/DataSource" );
        }
        catch ( NamingException e )
        {
            throw new EJBException( "EJB base class could not get JNDI context: " + e );
        }
    }

    /** Return a database connection from the database connection pool. <p>
    
        Close your connection when you are done with it! <p>
    */
    protected static Connection getConnection() throws Exception
    {
        return dataSource.getConnection();
    }

    /** SQL Helper: Get data from zero or more rows. <p>
    
        @param table The table to search and get data. <p>
        @param column The column to be searched - null gets all rows. <p>
        @param searchText The text to look for in column. <p>
        @param dataColumns The column names that have the data you want. <p>
        
        @return Each element in the ArrayList is an array of strings - each array contains the data for one row in the same order as in dataColumns. <p>
    */
    protected static ArrayList getRows( String table , String column , String searchText , String[] dataColumns )
    {
        ArrayList data = new ArrayList(); 
        try
        {
            Connection con = getConnection();
            try
            {
                Statement statement = con.createStatement();
                Str sql = new Str( "SELECT " );
                for( int i = 0 ; i < dataColumns.length ; i++ )
                {
                    sql.append( dataColumns[ i ] );
                    sql.append( ',' );
                }
                sql.deleteLast( 1 );
                sql.append( " FROM " );
                sql.append( table );
                if ( column != null )
                {
                    sql.append( " WHERE " );
                    sql.append( column );
                    sql.append( "='" );
                    sql.append( searchText );
                    sql.append( "'" );
                }
                try
                {
                    ResultSet rs = statement.executeQuery( sql.toString() );
                    while ( rs.next() )
                    {
                        String[] s = new String[ dataColumns.length ];
                        for( int i = 0 ; i < s.length ; i++ )
                        {
                            s[ i ] = rs.getString( dataColumns[ i ] );
                        }
                        data.add( s );
                    }
                }
                catch( Exception e )
                {
                    throw new EJBException( "getRow() execute fail: " + e );
                }
                finally
                {
                    statement.close();
                }
            }
            catch ( EJBException e )
            {
                throw e ;  // this is to avoid the exception message below
            }
            catch ( Exception e )
            {
                throw new EJBException( "getRow() create fail: " + e );
            }
            finally
            {
                con.close();
            }
        }
        catch ( EJBException e )
        {
            throw e ;  // this is to avoid the exception message below
        }
        catch( Exception e )
        {
            throw new EJBException( "getRow() connection fail: " + e );
        }
        return data ;
    }

    /** SQL Helper:  Get one row from a table. <p>
    
        @param table The table to search and get data. <p>
        @param column The column to be searched. <p>
        @param searchText The text to look for in column. <p>
        @param dataColumns The column names that have the data you want. <p>
        
        @return The data for one row in the same order as in dataColumns. <p>
    */
    protected static String[] getRow( String table , String column , String searchText , String[] dataColumns )
    {
        String[] returnVal = null ;
        ArrayList data = getRows( table , column , searchText , dataColumns );
        if ( data.size() == 1 )
        {
            returnVal = (String[])data.get( 0 );
        }
        return returnVal ;
    }

    /** SQL Helper: Find a row in a table and update selected fields. <p>
    
        @param table The table to search and update. <p>
        @param column The column to be searched. <p>
        @param searchText The text to look for in column. <p>
        @param data A key-value collection of column names (key) and data (value). If a column name starts with ">", that column will be processed as a stream (good for big Strings). <p>
    */
    protected static void setRow( String table , String column , String searchText , HashMap data )
    {
        try
        {
            Connection con = getConnection();
            try
            {
                Str sql = new Str( "UPDATE " + table + " SET " ); 
                ArrayList streamData = new ArrayList();
                Object[] keys = data.keySet().toArray();
                for ( int i = 0 ; i < keys.length ; i++ )
                {
                    String dataColumn = keys[ i ].toString();
                    String dataValue = data.get( dataColumn ).toString();
                    if ( dataColumn.charAt( 0 ) == '>' )
                    {
                        // insert data via a stream
                        dataColumn = dataColumn.substring( 1 , dataColumn.length() ); // delete first char
                        sql.append( dataColumn );
                        sql.append( "=?," );
                        streamData.add( dataValue );
                    }
                    else
                    {
                        // this data can be placed directly in the SQL
                        sql.append( dataColumn );
                        sql.append( "='" ); 
                        Str s = new Str( dataValue );
                        s.replace( '\'' , "''" );  // replace all single quotes with two single qutes
                        sql.append( s );
                        sql.append( "'," );
                    }
                }
                
                sql.deleteLast( 1 );
                
                sql.append( " WHERE " );
                sql.append( column );
                sql.append( "='" );
                sql.append( searchText );
                sql.append( "'" );
                
                PreparedStatement statement = con.prepareStatement( sql.toString() );
                
                for( int i = 0 ; i < streamData.size() ; i++ )
                {
                    String s = (String)streamData.get( i );
                    statement.setAsciiStream( i + 1 , new StringBufferInputStream( s ) , s.length() );
                }
                try
                {
                    statement.executeUpdate();
                }
                catch ( Exception e )
                {
                    throw new EJBException( "setCurrentRow() execute fail: " + e );
                }
                finally
                {
                    statement.close();
                }
            }
            catch ( EJBException e )
            {
                throw e ;  // this is to avoid the exception message below
            }
            catch ( Exception e )
            {
                throw new EJBException( "setCurrentRow() create fail: " + e );
            }
            finally
            {
                con.close();
            }
        }
        catch ( EJBException e )
        {
            throw e ;  // this is to avoid the exception message below
        }
        catch( Exception e )
        {
            throw new EJBException( "setCurrentRow() connection fail: " + e );
        }
    }
    
    /** SQL Helper: Search table and delete matches. <p>
    
        @param table The table to search and delete. <p>
        @param column The column to be searched. <p>
        @param searchText The text to look for in column. <p>
    */
    protected static void deleteRows( String table , String column , String searchText )
    {
        try
        {
            Connection con = getConnection();
            try
            {
                Statement statement = con.createStatement();
                String sql = "DELETE FROM " + table + " WHERE " + column + "='" + searchText + "'" ;
                try
                {
                    statement.executeUpdate( sql );
                }
                catch( Exception e )
                {
                    throw new EJBException( "deleteRows() execute fail: " + e );
                }
                finally
                {
                    statement.close();
                }
            }
            catch ( EJBException e )
            {
                throw e ;  // this is to avoid the exception message below
            }
            catch ( Exception e )
            {
                throw new EJBException( "deleteRows() create fail: " + e );
            }
            finally
            {
                con.close();
            }
        }
        catch ( EJBException e )
        {
            throw e ;  // this is to avoid the exception message below
        }
        catch( Exception e )
        {
            throw new EJBException( "deleteRows() connection fail: " + e );
        }
    }

    /** Override this method only if you need to. <p>
    */
    public void ejbActivate(){}
    
    /** Override this method only if you need to. <p>
    */
    public void ejbPassivate(){}
    
}

