package com.javaranch.common ;

import java.util.* ;
import java.io.* ;
import java.sql.* ;
import javax.ejb.* ;
import javax.naming.* ;
import javax.sql.* ;

/** This class provides empty EntityBean methods and a collection of BMP/SQL helper methods. <p>

    This class is intended for use inside an EJB server only! <p> 
    
    
    - - - - - - - - - - - - - - - - - <p>
    
    Copyright (c) 1998-2000 Paul Wheaton <p>
    
    You are welcome to do whatever you want to with this source file provided
    that you maintain this comment fragment (between the dashed lines).
    Modify it, change the package name, change the class name ... personal or business 
    use ...  sell it, share it ... add a copyright for the portions you add ... <p>
    
    My goal in giving this away and maintaining the copyright is to hopefully direct 
    developers back to JavaRanch. <p>
    
    The original source can be found at <a href=http://www.javaranch.com>JavaRanch</a> <p>
    
    - - - - - - - - - - - - - - - - - <p>
    
    @author Paul Wheaton
*/
public abstract class EntityEJB extends EJB implements EntityBean
{

    protected EntityContext ctx ;
    private boolean dirty = false ;
    
    /** You must override this method and provide the name of the table that the primary key is in. <p>
    */
    protected abstract String getPKTable();

    /** You must override this method and provide the name of the column that the primary key is in. <p>
    */
    protected abstract String getPKColumn();

    /** Implemented for you - you can access the context as "ctx". <p>  
        
        If you override this method, you must set ctx appropriately or other methods in this class won't work correctly. <p>
    */                                   
    public void setEntityContext( EntityContext ctx )
    {
        this.ctx = ctx ;
    }

    /** Implemented for you - "ctx" is set to null. <p>  
    */                                   
    public void unsetEntityContext()
    {
        ctx = null ;
    }
    
    /** When you have stuff to be written to the database, call setDirty( true ). <p>
    
        Pass in true whenever you just did something that will eventually need to be written to the 
        database.  When everything has been written to the database, "dirty" will be set to false. <p>
        
        
        If you are using BMP, make sure at the end of your methods ejbCreate(), ejbLoad() and ejbStore()
        you call setDirty( false ). <p>
    */
    protected void setDirty( boolean b )
    {
        dirty = b ;   
    }
    
    /** Call this during ejbStore() to find out if you should write; write only if true. <p>
    */
    protected boolean isDirty()
    {
        return dirty ;
    }
    
    /** You must override this method to properly store all of your bean data. <p>
    
        Be sure that you use the dirty flag and the setCurrentRow() method properly.<p>
        
        Example:<p>
        
        <pre>
        
    *    public void ejbStore()   
    *    {
    *        if ( isDirty() )
    *        {
    *            HashMap data = new HashMap();
    *            data.put( "COLUMN1" , attribute1 );
    *            data.put( "COLUMN2" , attribute2 );
    *            setCurrentRow( data );
    *            setDirty( false );
    *        }
    *    }
        
        </pre>
    
    */
    public abstract void ejbStore();

    /** You must override this method to properly load all of your bean data. <p>
    
        Be sure that you use the dirty flag and the getCurrentRow() method properly.<p>
        
        Example:<p>
        
        <pre>
        
    *    private static final String[] dataColumns = { "COLUMN1" , "COLUMN2" };
    *    
    *    public void ejbLoad()
    *    {
    *        String[] data = getCurrentRow( columnData );
    *        attribute1 = data[0];
    *        attribute2 = data[1];
    *        setDirty( false );
    *    }
        
        </pre>
    */
    public abstract void ejbLoad();

    /** Search the PK table and return all of the matching primary keys as Strings. <p>
    
        Useful in finder methods. <p>

        Example: <p>
        
        <pre>

    *    public Enumeration ejbFindByLastName( String lastName )
    *    {
    *        return Collections.enumeration( getPrimaryKeys( "FILENAME" , filename ) );
    *    }
        
        </pre>
    
        @param column The column to be searched. <p>
        @param searchText The text to search for in column. <p>
        
        @return A collection of strings representing the primary key. <p>
    */
    protected ArrayList getPrimaryKeys( String column , String searchText )
    {
        String[] columns = new String[ 1 ];
        columns[0] = getPKColumn();
        ArrayList data = getRows( getPKTable() , column , searchText , columns );
        ArrayList returnVal = new ArrayList();
        for ( int i = 0 ; i < data.size() ; i++ )
        {
            String[] s = (String[])data.get( i );
            returnVal.add( s[0] );   
        }
        return returnVal ;
    }
    
    /** Search the PK table on the PK column and return the one matching primary key as a string. 

        Useful for the ejbFindByPrimaryKey() method. <p>
        
        The primary purpose of this method is to throw FinderException if the key cannot be found. <p>

        Example: <p>
        
        <pre>

    *    public Integer ejbFindByPrimaryKey( Integer key ) throws FinderException
    *    {
    *        getPrimaryKey( key.toString() );
    *        return key ;
    *    }

        </pre>
    
        @param searchText The text to search for in the PK column. <p>
        
        @return The found primary key. <p>
    */
    protected String getPrimaryKey( String searchText ) throws FinderException
    {
        String returnVal = null ;
        ArrayList data = getPrimaryKeys( getPKColumn() , searchText );
        if ( data.size() == 1 )
        {
            returnVal = (String)data.get(0);
        }
        else
        {
            throw new ObjectNotFoundException( "getPrimaryKey() cannot find " + searchText );
        }
        return returnVal ;
    }
    
    /** The PK table is searched for the current PK in the PK column and that row is deleted. <p>
    */
    protected void deleteCurrentRow()
    {
        deleteRows( getPKTable() , getPKColumn() , ctx.getPrimaryKey().toString() );
    }

    /** Implemented for you - calls deleteCurrentRow(). <p>
    */
    public void ejbRemove()
    {
        deleteCurrentRow();        
    }
    
    /** The PK table is searched for the current PK in the PK column and that row is loaded. <p>
    
        Useful when implementing ejbLoad(). <p>
        
        Example:<p>
        
        <pre>
        
    *    private static final String[] dataColumns = { "COLUMN1" , "COLUMN2" };
    *    
    *    public void ejbLoad()
    *    {
    *        String[] data = getCurrentRow( columnData );
    *        attribute1 = data[0];
    *        attribute2 = data[1];
    *        setDirty( false );
    *    }
        
        </pre>
        
        @param dataColumns The column names that have the data you want. <p>
        
        @return The data for one row in the same order as in dataColumns. <p>
    */
    protected String[] getCurrentRow( String[] dataColumns )
    {
        return getRow( getPKTable() , getPKColumn() , ctx.getPrimaryKey().toString() , dataColumns );
    }

    /** The PK table is searched for the current PK in the PK column and that row is loaded. <p>
    
        Useful when implementing ejbStore().<p>
        
        Example:<p>
        
        <pre>
        
    *    public void ejbStore()   
    *    {
    *        if ( isDirty() )
    *        {
    *            HashMap data = new HashMap();
    *            data.put( "COLUMN1" , attribute1 );
    *            data.put( "COLUMN2" , attribute2 );
    *            setCurrentRow( data );
    *            setDirty( false );
    *        }
    *    }
        
        </pre>
    
        @param data A key-value collection of column names (key) and data (value). If a column name starts with ">", that column will be processed as a stream (good for big Strings). <p>
    */
    protected void setCurrentRow( HashMap data )
    {
        setRow( getPKTable() , getPKColumn() , ctx.getPrimaryKey().toString() , data );
    }
    
    /** Create a new row with a unique ID. <p>

        Useful when implementing ejbCreate().<p>
        
        Example: <p>
        
        <pre>
    
    *    public Integer ejbCreate( String data1 , String data2 ) throws CreateException
    *    {
    *        this.data1 = data1 ;
    *        this.data2 = data2 ;
    *        
    *        HashMap data = new HashMap();
    *        data.put( "COLUMN1" , data1 );
    *        data.put( "COLUMN2" , data2 );
    *        id = newRowWithID( data );
    *
    *        setDirty( false );
    *        return new Integer( id );
    *    }
        
        </pre>
        
        Requires that a sequence is set up in the database called "TABLENAME_SEQ". <p>

        @param data A key-value collection of column names (key) and data (value). If a column name starts with ">", that column will be processed as a stream (good for big Strings). <p>
        
        @return The new ID. <p>
    */
    protected int newRowWithID( HashMap data ) throws CreateException
    {
        int id = -1 ;
        try
        {
            Connection con = getConnection();
            try
            {
                // this try block is required because of the finally at the bottom to close the connection
                
                // get unique ID
                try
                {
                    Statement statement = con.createStatement();
                    try
                    {
                        ResultSet rs = statement.executeQuery( "select " + getPKTable() + "_seq.nextval from dual" );
                        rs.next();
                        id = rs.getInt("NEXTVAL");
                    }
                    catch ( Exception e )
                    {
                        throw new CreateException( "newRowWithID() NEXTVAL fail: " + e );
                    }
                    finally
                    {
                        statement.close();
                    }
                }
                catch ( CreateException e )
                {
                    throw e ;  // this is to avoid the exception message below
                }
                catch( Exception e )
                {
                    throw new CreateException( "newRowWithID() NEXTVAL create fail: " + e );
                }
                
                // create row
                try
                {
                    Str columns = new Str( getPKColumn() + ',' );
                    Str values = new Str( "'" + id + "'," );
                    ArrayList streamData = new ArrayList();
                    Object[] keys = data.keySet().toArray();
                    for ( int i = 0 ; i < keys.length ; i++ )
                    {
                        String dataColumn = keys[ i ].toString();
                        String dataValue = data.get( dataColumn ).toString();
                        if ( dataColumn.charAt( 0 ) == '>' )
                        {
                            // we need to mash this data in via a stream
                            dataColumn = dataColumn.substring( 1 , dataColumn.length() );
                            columns.append( dataColumn );
                            columns.append( ',' );
                            values.append( "?," );
                            streamData.add( dataValue );
                        }
                        else
                        {
                            // this data can be placed directly in the SQL
                            columns.append( dataColumn );
                            columns.append( ',' ); 
                            values.append( "'" );
                            Str s = new Str( dataValue );
                            s.replace( '\'' , "''" );
                            values.append( s );
                            values.append( "'," );
                        }
                    }
                    columns.deleteLast( 1 );
                    values.deleteLast( 1 );

                    String sql = "INSERT INTO " + getPKTable() + " (" + columns + ") VALUES (" + values + ")" ; 
                    
                    PreparedStatement statement = con.prepareStatement( sql );
                    
                    for( int i = 0 ; i < streamData.size() ; i++ )
                    {
                        String s = (String)streamData.get( i );
                        statement.setAsciiStream( i + 1 , new StringBufferInputStream( s ) , s.length() );
                    }
                    try
                    {
                        statement.executeUpdate();
                    }
                    catch ( Exception e )
                    {
                        throw new CreateException( "newRowWithID() insert fail: " + e );
                    }
                    finally
                    {
                        statement.close();
                    }
                }
                catch ( CreateException e )
                {
                    throw e ;  // this is to avoid the exception message below
                }
                catch ( Exception e )
                {
                    throw new CreateException( "newRowWithID() create fail: " + e );
                }
            }
            finally
            {
                con.close();
            }
        }
        catch ( CreateException e )
        {
            throw e ;  // this is to avoid the exception message below
        }
        catch( Exception e )
        {
            throw new CreateException( "newRowWithID() connection fail: " + e );
        }
        return id ;
    }

}
