package com.javaranch.common ;

import java.util.* ;
import java.io.* ;
import java.text.* ;
import java.lang.reflect.* ;


/** Stand alone program to execute all unitTest() methods in a com tree. <p>

    This program will recursively scan a com tree for all class files.  Every class file
    will be loaded.  If the class file is not an interface or an anonymous class, the unitTest()
    method will be invoked.  If the unitTest() method is not found, it will be reported. <p>

    A proper unitTest() method will have the following header: <p>
    <ul>
      public static void unitTest( ErrorLog log )
    </ul>
    <p>

    The ErrorLog object can be found in com.digitalglobe.common. <p>

    For every unit test failure, call log.add("my failure message"); <p>

    Any messages added to the log will have a time/date stamp along with the class name added. <p>

    If you activate this program without passing any parameters, it will try to find the com
    tree in the current directory.  If it cannot find the com tree, it will try to find class
    files in the current directory. <p>

    You can pass a parameter that represents the directory where the com tree can be found.  If
    a directory is not found, this program will search for classes in that directory. <p>

    A parameter can specify an exact file, in which case only that one file is processed.
    Wildcards are not accepted. <p>


    Suggested practices: <p>

    <ul>

      When any bug is discovered, add a test to your unitTest() that exposes the bug.  Fix the bug.
      Later regression testing will now make sure that the bug doesn't come back. <p>

    </ul>
    
    - - - - - - - - - - - - - - - - - <p>
    
    Copyright (c) 1999-2000 Paul Wheaton <p>
    Copyright (c) 1999-2000 EarthWatch, Inc.
    
    You are welcome to do whatever you want to with this source file provided
    that you maintain this comment fragment (between the dashed lines).
    Modify it, change the package name, change the class name ... personal or business 
    use ...  sell it, share it ... add a copyright for the portions you add ... <p>
    
    My goal in giving this away and maintaining the copyright is to hopefully direct 
    developers back to JavaRanch. <p>
    
    I originally developed this class while working as a contractor at EarthWatch, Inc. in
    Longmont, Colorado.  They gave me permission to distribute this code this way provided
    that their message would also be carried along.  Their message is that they hire
    Java programmers and would like you to consider working with them.  I have to say
    that my experience with them was first rate and I would encourage engineers to work there.
    Check out their web site at <a href=http://www.digitalglobe.com>http://www.digitalglobe.com</a>. <p>
    
    The original source can be found at <a href=http://www.javaranch.com>JavaRanch</a> <p>
    
    - - - - - - - - - - - - - - - - - <p>

    @author Paul Wheaton

*/

public class UnitTest
{

    private static TestLog log ;
    private static int pass = 0 ;
    private static int fail = 0 ;
    private static boolean treeMode = true ; // go through the com tree, or do just this directory

    private static class TestLog extends ErrorLog
    {

        private ErrorLog summary ;
        private String prefix ;

        TestLog()
        {
            summary = new ErrorLog();
            summary.setInternalLog( true );
            summary.setConsole( false );
            SimpleDateFormat f = new SimpleDateFormat("_yyyyMMdd_HHmmss");
            String fileName = "UnitTestLog" + f.format( new Date() ) + ".txt" ;
            try
            {
                setLogFile( fileName );
            }
            catch( IOException e )
            {
                String message = "could not open log file " + fileName ;
                System.out.println( message );
                summary.add( message );
            }
        }

        public void add( String message )
        {
            String s = prefix + message ;
            super.add( s );
            summary.add( s );
        }

        void setClassName( String currentClassName )
        {
            prefix = "(" + currentClassName + ")" ;
        }

        void startMessage()
        {
            super.add( prefix + "start" );
        }

        void stopMessage()
        {
            super.add( prefix + "complete" );
        }

        void anonymousSkipMessage()
        {
            super.add( prefix + "anonymous inner class - skipping" );
        }

        void interfaceSkipMessage()
        {
            super.add( prefix + "interface - skipping" );
        }

        void showSummary()
        {
            String[] s = summary.getList();
            Str bigString = new Str( "\n\nSummary of failures:\n" ) ;
            for( int i = 0 ; i < s.length ; i++ )
            {
                bigString.append( "\n    " + s[ i ] );
            }
            bigString.append("\n\nclasses passed: " + pass + "\n\nclassed failed: " + fail );
            super.add( bigString.toString() );
        }

    }

    private static void test( String className )
    {
        log.setClassName( className );
        log.startMessage();
        int logCount = log.numErrors();
        try
        {
            Class c = Class.forName( className );
            if ( c.isInterface() )
            {
                log.interfaceSkipMessage();
            }
            else
            {
                Method m = c.getDeclaredMethod( "unitTest" , new Class[]{ ErrorLog.class } );
                m.invoke( null , new Object[]{ log } );
            }
        }
        catch( NoClassDefFoundError e )
        {
            log.add( "class is not in proper package" );
        }
        catch( NoSuchMethodException e )
        {
            log.add( "could not find unitTest() method" );
        }
        catch( Throwable e )
        {
            log.add( "exception encountered during unit test: " + e );
        }
        if ( log.numErrors() > logCount )
        {
            fail++ ;
        }
        else
        {
            pass++ ;
        }
        log.stopMessage();
    }

    private static void processDirectory( File dir , String packageName )
    {
        System.out.println( "processing package " + packageName );
        String[] files = dir.list();
        for( int i = 0 ; i < files.length ; i++ )
        {
            String name = files[ i ];
            String fullPackageName = packageName + "." + name ;
            File f = new File( dir , name );
            if ( f.isDirectory() )
            {
                if ( treeMode )
                {
                    processDirectory( f , fullPackageName );
                }
            }
            else if ( name.endsWith( ".class" ) )
            {
                Str s = new Str( fullPackageName );
                s.trimTrailTo( '.' );
                boolean anon = false ;
                int trailingDigits = s.trailingDigits();
                if ( trailingDigits > 0 )
                {
                    if ( s.charAt( s.length() - ( trailingDigits + 1 ) ) == '$' )
                    {
                        anon = true ;
                    }
                }
                String className = s.toString();
                if ( anon )
                {
                    log.setClassName( className );
                    log.anonymousSkipMessage();
                }
                else
                {
                    test( className );
                }
            }
        }
    }

    private static String getPackage( File f )
    {
        Str s = new Str( f.getAbsolutePath() );
        s.replace( '/' , '.' );
        s.replace( '\\' , '.' );
        while( s.endsWith( "." ) )
        {
            s.setLength( s.length() - 1 );
        }
        int dotCom = s.indexOf( ".com." );
        return s.after( dotCom ).toString();
    }

    public static void main( String[] args ) throws Exception
    {
        log = new TestLog();
        boolean didTesting = false ;

        String path = ".";
        if ( args.length == 1 )
        {
            path = args[0];
        }
        File comDir = new File( path + "/com" );
        if ( comDir.exists() )
        {
            if ( comDir.isDirectory() )
            {
                processDirectory( comDir , "com" );
                didTesting = true ;
            }
            else
            {
                System.out.println( "'com' is not a directory" );
            }
        }
        else
        {
            File f = new File( path );
            if ( ! f.exists() )
            {
                f = new File( path + ".class" );
            }
            if ( f.exists() )
            {
                String packageName = getPackage( f );
                if ( f.isDirectory() )
                {
                    treeMode = false ;
                    processDirectory( f , packageName );
                    didTesting = true ;
                }
                else
                {
                    Str s = new Str( packageName );
                    s.trimTrailTo( '.' );
                    test( s.toString() );
                    didTesting = true ;
                }
            }
            else
            {
                System.out.println( "could not determine file or directory." );
            }
        }
        if ( didTesting )
        {
            log.showSummary();
            System.exit(0); // we may have loaded some stuff that kicked off a thread or two.
        }
    }

}



